library(tidyverse)
library(plotly)
library(ggpubr)
n.rep <- 100 # number of simulated data sets
cores <- 100 # number of cores in parallel
n.strata <- 1 # number of strata
mean.size.strata <- 10000 # average size of strata
# size.strata <- rpois(n.strata, mean.size.strata) # can be user-specified
size.strata <- rep(mean.size.strata, n.strata) # user-specified
rho <- 0.6 # correlation of AR(1) covar structure
p <- 2 # number of covariates
Z.char <- paste0("z",1:p) # covariate names
k.pd <- 7 # number of knots in B-spline 6
k.cal <- 7
degree.pd <- 3 # 2
degree.cal <- 3
end.pd <- 3 # end-of-followup time
end.cal <- 5
set.seed(1234)
level <- 0.95
quant.upper <- qnorm((1+level)/2)
strata <- rep(letters[1:n.strata], size.strata) # strata IDs
times.pd <- seq(0, end.pd, end.pd/100)
times.cal <- seq(0, end.cal, end.cal/100)
beta.true <- matrix(nrow=length(times.cal), ncol=length(times.pd))
for (i in 1:length(times.cal)) {
  for (j in 1:length(times.pd)) {
    beta.true[i, j] <- exp(-0.5*times.cal[i])*sin(3*pi*times.pd[j]/4)
  }
}
D1 <- diff(diag(k.cal)); D2 <- diff(diag(k.pd))
mu1 <- 0.0005; mu2 <- 0.0002
cl <- parallel::makeCluster(cores)
doParallel::registerDoParallel(cl)
`%dorng%` <- doRNG::`%dorng%`
`%dopar%` <- foreach::`%dopar%`
system.time(raw <-
              foreach::foreach(i=1:n.rep, .verbose=F, #.errorhandling="remove", 
                               .options.RNG=1234,
                               .packages=c("mvtnorm","survival","Rcpp","RcppArmadillo"),
                               #.combine=rbind,
                               .noexport=c("surtiver_fixtra_bresties_fit_diffpen")) 
            %dorng% {
              Rcpp::sourceCpp("surtiver_tps.cpp")
              gamma <- rep(rnorm(n.strata, sd=0.5), size.strata) # strata-specific effects
              Z <- mvtnorm::rmvnorm(sum(size.strata),
                                    sigma=rho^abs(outer(1:p, 1:p, "-"))) # covariates
              ts.cal <- round(runif(sum(size.strata), 0, end.cal), 5)
              U <- runif(sum(size.strata)) # survivor function F(t)
              integrand <- function(t, gamma, z, t.cal) { # gamma is a scalar, z is a row of Z
                return(c(0.5*exp(gamma+z[1]+exp(-0.5*t.cal)*sin(3*pi*t/4)*(t<=end.pd)*z[2])))
              }
              time.failure <- sapply(1:sum(size.strata), function(i) {
                res <- try(uniroot(function(t) integrate(Vectorize(function(x)
                  integrand(x, gamma[i], Z[i,], ts.cal[i])), 0, t)$value +
                    log(1 - U[i]), c(0, end.pd+1)), silent=T)
                return(ifelse(class(res)=="try-error", end.pd, res$root))
              })
              time.censoring <- runif(sum(size.strata), 0, end.pd)
              data <- data.frame(time.pd=round(pmin(time.failure, time.censoring), 5),
                                 time.cal=ts.cal,
                                 event=1*(time.failure<=time.censoring),
                                 strata, Z, stringsAsFactors=F)
              names(data)[-(1:4)] <- Z.char
              data <- data[order(data$strata, data$time.pd, -data$event),]
              knots.pd <- quantile(data$time.pd[data$event==1],
                                   probs=(1:(k.pd-degree.pd-1))/(k.pd-degree.pd))
              uniqt.pd.str <- unique(data$time.pd[data$event==1])
              bases.pd <- splines::bs(uniqt.pd.str, intercept=T,
                                      knots=knots.pd, degree=degree.pd,
                                      Boundary.knots=c(0, end.pd))
              knots.cal <- quantile(data$time.cal[data$event==1],
                                    probs=(1:(k.cal-degree.cal-1))/(k.cal-degree.cal))
              bases.cal <- splines::bs(data$time.cal, intercept=T,
                                       knots=knots.cal, degree=degree.cal,
                                       Boundary.knots=c(0, end.cal))
              count.strata <- sapply(split(data$strata, data$strata), length)
              theta.init <- matrix(runif(1*k.pd*k.cal), 1, k.pd*k.cal)
              fit.ProxN <-
                surtiver_fixtra_bresties_fit_diffpen(data$event, data$time.pd,
                                                     count.strata, 
                                                     as.matrix(data[,Z.char[2]]),
                                                     as.matrix(bases.cal),
                                                     as.matrix(bases.pd), 
                                                     theta.init,
                                                     as.matrix(data[,Z.char[1]]), 
                                                     as.numeric(0),
                                                     D1, D2, mu1, mu2,
                                                     method="ProxN")
              B.cal <- as.matrix(splines::bs(times.cal, degree=degree.cal,
                                             intercept=T, knots=knots.cal,
                                             Boundary.knots=c(0, end.cal)))
              B.pd <- splines::bs(times.pd, degree=degree.pd, intercept=T,
                                  knots=knots.pd, Boundary.knots=c(0, end.pd))
              beta.est <- B.cal%*%matrix(fit.ProxN$ctrl.pts[1,], byrow=T,
                                         ncol=k.pd)%*%t(B.pd)
              beta.est
            })
parallel::stopCluster(cl)
save(raw, file="list_of_matrices_n=10000_pen_mu1_0.0005_mu2_0.0002.RData")
save(raw, file="list_of_matrices_n=10000_pen_mu1_0.005_mu2_0.002.RData")
save(raw, file="list_of_matrices_n=10000_pen_mu1_0.05_mu2_0.02.RData")
mytheme <- theme(plot.title=element_text(hjust=0),
                 axis.title=element_text(size=14),
                 axis.text=element_text(size=12),
                 text=element_text(size=12),
                 legend.text=element_text(size=14),
                 legend.title=element_blank(),
                 strip.text=element_text(size=14))

pmean <- Reduce("+", raw)/n.rep
mse <- (lapply(raw, function(mat) (mat-beta.true)^2) %>% Reduce("+", .))/n.rep
bias <- ((lapply(raw, function(mat) mat-beta.true) %>% Reduce("+", .))/n.rep)^2
variance <- (lapply(raw, function(mat) (mat-pmean)^2) %>% Reduce("+", .))/n.rep

df.pd <- tibble(`time to event`=times.pd, IMSE=colMeans(mse), 
                `squared bias`=colMeans(bias), variance=colMeans(variance)) %>%
  pivot_longer(cols=c("IMSE", "squared bias", "variance"), names_to="measure", values_to="value")
df.cal <- tibble(`calendar time`=times.cal, IMSE=rowMeans(mse), 
                 `squared bias`=rowMeans(bias), variance=rowMeans(variance)) %>%
  pivot_longer(cols=c("IMSE", "squared bias", "variance"),
               names_to="measure", values_to="value")

df.pd.10000 <- df.pd
df.cal.10000 <- df.cal
df.pd.1000 <- df.pd
df.cal.1000 <- df.cal
df.pd.100 <- df.pd
df.cal.100 <- df.cal

df.pd.100 <- df.pd.100 %>% mutate(mu1=5, mu2=2) # 0.05*100 and 0.02*100 due to sample size=10,000
df.pd.1000 <- df.pd.1000 %>% mutate(mu1=0.5, mu2=0.2)
df.pd.10000 <- df.pd.10000 %>% mutate(mu1=0.05, mu2=0.02)

df.cal.100 <- df.cal.100 %>% mutate(mu1=5, mu2=2)
df.cal.1000 <- df.cal.1000 %>% mutate(mu1=0.5, mu2=0.2)
df.cal.10000 <- df.cal.10000 %>% mutate(mu1=0.05, mu2=0.02)


df.pd <- df.pd.100 %>% bind_rows(df.pd.1000, df.pd.10000)
df.cal <- df.cal.100 %>% bind_rows(df.cal.1000, df.cal.10000)
df.pd <- df.pd %>% rename(`event time`=`time to event`)
df.pd <- df.pd %>% mutate(`event time`=`event time`*10)
df.cal <- df.cal %>% mutate(`calendar time`=`calendar time`*10)

df.pd <- df.pd %>% mutate(tuning=paste0("mu1=", df.pd %>% pull(mu1), ", mu2=", df.pd %>% pull(mu2)))
df.cal <- df.cal %>% mutate(tuning=paste0("mu1=", df.cal %>% pull(mu1), ", mu2=", df.cal %>% pull(mu2)))

save(df.pd, df.cal, file="acc_pd_cal_pen.RData")

IMSE.pd <- df.pd %>% filter(measure=="IMSE") %>%
  filter(`event time`<=27) %>%
  ggplot(aes(x=`event time`, y=value, group=tuning, color=tuning, linetype=tuning)) +
  geom_line() +
  scale_y_continuous("IMSE", limits=c(0,1)) +
  scale_color_manual(values=c("black", "blue", "red")) +
  scale_linetype_manual(values=c("dotdash", "dotted", "solid")) +
  theme_classic() + mytheme

bias.pd <- df.pd %>% filter(measure=="squared bias") %>%
  filter(`event time`<=27) %>%
  ggplot(aes(x=`event time`, y=sqrt(value), group=tuning, color=tuning, linetype=tuning)) +
  geom_line() +
  scale_y_continuous("Bias", limits=c(0,1)) +
  scale_color_manual(values=c("black", "blue", "red")) +
  scale_linetype_manual(values=c("dotdash", "dotted", "solid")) +
  theme_classic() + mytheme

var.pd <- df.pd %>% filter(measure=="variance") %>%
  filter(`event time`<=27) %>%
  ggplot(aes(x=`event time`, y=value, group=tuning, color=tuning, linetype=tuning)) +
  geom_line() +
  scale_y_continuous("variance", limits=c(0,1)) +
  scale_color_manual(values=c("black", "blue", "red")) +
  scale_linetype_manual(values=c("dotdash", "dotted", "solid")) +
  theme_classic() + mytheme

IMSE.cal <- df.cal %>% filter(measure=="IMSE") %>%
  ggplot(aes(x=`calendar time`, y=value, group=tuning, color=tuning, linetype=tuning)) +
  geom_line() +
  scale_y_continuous("IMSE", limits=c(0,1)) +
  scale_color_manual(values=c("black", "blue", "red")) +
  scale_linetype_manual(values=c("dotdash", "dotted", "solid")) +
  theme_classic() + mytheme

bias.cal <- df.cal %>% filter(measure=="squared bias") %>%
  ggplot(aes(x=`calendar time`, y=sqrt(value), group=tuning, color=tuning, linetype=tuning)) +
  geom_line() +
  scale_y_continuous("Bias", limits=c(0,1)) +
  scale_color_manual(values=c("black", "blue", "red")) +
  scale_linetype_manual(values=c("dotdash", "dotted", "solid")) +
  theme_classic() + mytheme

var.cal <- df.cal %>% filter(measure=="variance") %>%
  ggplot(aes(x=`calendar time`, y=value, group=tuning, color=tuning, linetype=tuning)) +
  geom_line() +
  scale_y_continuous("variance", limits=c(0,1)) +
  scale_color_manual(values=c("black", "blue", "red")) +
  scale_linetype_manual(values=c("dotdash", "dotted", "solid")) +
  theme_classic() + mytheme

# pen and unpen for n=10000
load("acc_pd_cal.RData")
df.cal.unpen <- df.cal
df.pd.unpen <- df.pd
load("acc_pd_cal_pen.RData")
df.cal.pen <- df.cal
df.pd.pen <- df.pd

df.cal <- df.cal.unpen %>% filter(`sample size`==10000) %>%
  select(-`sample size`) %>%
  mutate(tuning="mu1=0, mu2=0") %>%
  bind_rows(df.cal.pen %>% select(-c(mu1, mu2)))

df.pd <- df.pd.unpen %>% filter(`sample size`==10000) %>%
  select(-`sample size`) %>%
  mutate(tuning="mu1=0, mu2=0") %>%
  bind_rows(df.pd.pen %>% select(-c(mu1, mu2)))

mytheme <- theme(plot.title=element_text(hjust=0),
                 axis.title=element_text(size=14),
                 axis.text=element_text(size=12),
                 text=element_text(size=12),
                 legend.text=element_text(size=12),
                 legend.title=element_blank(),
                 strip.text=element_text(size=14))

IMSE.pd <- df.pd %>% filter(measure=="IMSE") %>%
  filter(`event time`<=27) %>%
  ggplot(aes(x=`event time`, y=value, group=tuning, color=tuning, linetype=tuning)) +
  geom_line() +
  scale_y_continuous("IMSE", limits=c(0,1)) +
  scale_color_manual(values=c("red", "blue", "black", "green")) +
  scale_linetype_manual(values=c("dotted", "solid", "dotdash", "dashed")) +
  theme_classic() + mytheme

bias.pd <- df.pd %>% filter(measure=="squared bias") %>%
  filter(`event time`<=27) %>%
  ggplot(aes(x=`event time`, y=sqrt(value), group=tuning, color=tuning, linetype=tuning)) +
  geom_line() +
  scale_y_continuous("Bias", limits=c(0,1)) +
  scale_color_manual(values=c("red", "blue", "black", "green")) +
  scale_linetype_manual(values=c("dotted", "solid", "dotdash", "dashed")) +
  theme_classic() + mytheme

var.pd <- df.pd %>% filter(measure=="variance") %>%
  filter(`event time`<=27) %>%
  ggplot(aes(x=`event time`, y=value, group=tuning, color=tuning, linetype=tuning)) +
  geom_line() +
  scale_y_continuous("variance", limits=c(0,1)) +
  scale_color_manual(values=c("red", "blue", "black", "green")) +
  scale_linetype_manual(values=c("dotted", "solid", "dotdash", "dashed")) +
  theme_classic() + mytheme

IMSE.cal <- df.cal %>% filter(measure=="IMSE") %>%
  ggplot(aes(x=`calendar time`, y=value, group=tuning, color=tuning, linetype=tuning)) +
  geom_line() +
  scale_y_continuous("IMSE", limits=c(0,1)) +
  scale_color_manual(values=c("red", "blue", "black", "green")) +
  scale_linetype_manual(values=c("dotted", "solid", "dotdash", "dashed")) +
  theme_classic() + mytheme

bias.cal <- df.cal %>% filter(measure=="squared bias") %>%
  ggplot(aes(x=`calendar time`, y=sqrt(value), group=tuning, color=tuning, linetype=tuning)) +
  geom_line() +
  scale_y_continuous("Bias", limits=c(0,1)) +
  scale_color_manual(values=c("red", "blue", "black", "green")) +
  scale_linetype_manual(values=c("dotted", "solid", "dotdash", "dashed")) +
  theme_classic() + mytheme

var.cal <- df.cal %>% filter(measure=="variance") %>%
  ggplot(aes(x=`calendar time`, y=value, group=tuning, color=tuning, linetype=tuning)) +
  geom_line() +
  scale_y_continuous("variance", limits=c(0,1)) +
  scale_color_manual(values=c("red", "blue", "black", "green")) +
  scale_linetype_manual(values=c("dotted", "solid", "dotdash", "dashed")) +
  theme_classic() + mytheme

ggarrange(IMSE.pd, IMSE.cal, bias.pd, bias.cal, var.pd, var.cal, common.legend=T, ncol=2, nrow=3)
ggsave(file=paste0("acc_pd_cal_pen_unpen_n=10000_", Sys.Date(), ".pdf"),
       width=7, height=9)

load("list_of_matrices_n=10000_pen_mu1_0.005_mu2_0.002.RData")
idx <- c(21,41,61,81)
accplt.pd <- lapply(idx, function(id) {
  sapply(raw, function(mat) mat[id,]) %>%
    {tibble(
      `calendar time`=times.cal[id]*10,
      `event time`=times.pd*10,
      true=as.numeric(beta.true[id,]),
      mean=as.numeric(rowMeans(.)),
      lower=as.numeric(apply(., 1, function(r) quantile(r, 0.025))),
      upper=as.numeric(apply(., 1, function(r) quantile(r, 0.975))))}
}) %>% bind_rows %>% mutate(`calendar time`=factor(`calendar time`)) %>%
  ggplot(aes(x=`event time`)) +
  geom_line(aes(y=true, linetype="true value"), size=0.9) +
  geom_line(aes(y=mean, linetype="mean"), size=0.9) +
  geom_ribbon(aes(ymin=lower, ymax=upper,
                  fill="95% percentile range"), alpha=0.4) +
  scale_x_continuous(expand=c(1,1)/100) +
  scale_y_continuous(name="log hazard ratio", expand=c(1,1)/100) +
  scale_linetype_manual("", values=c("mean"="solid", "true value"="dotted")) +
  scale_fill_manual("", values="grey") +
  guides(linetype=guide_legend(nrow=1), fill=guide_legend(nrow=1)) +
  facet_grid(`calendar time`~., labeller=label_both) +
  theme_classic() + mytheme +
  theme(plot.title=element_text(hjust=0),
        legend.position="top", legend.box="horizontal")

accplt.cal <- lapply(idx, function(id) {
  sapply(raw, function(mat) mat[,id]) %>%
    {tibble(
      `calendar time`=times.cal*10,
      `event time`=times.pd[id]*10,
      true=as.numeric(beta.true[,id]),
      mean=as.numeric(rowMeans(.)),
      lower=as.numeric(apply(., 1, function(r) quantile(r, 0.025))),
      upper=as.numeric(apply(., 1, function(r) quantile(r, 0.975))))}
}) %>% bind_rows %>% mutate(`event time`=factor(`event time`)) %>%
  ggplot(aes(x=`calendar time`)) +
  geom_line(aes(y=true, linetype="true value"), size=0.9) +
  geom_line(aes(y=mean, linetype="mean"), size=0.9) +
  geom_ribbon(aes(ymin=lower, ymax=upper,
                  fill="95% percentile range"), alpha=0.4) +
  scale_x_continuous(expand=c(1,1)/100) +
  scale_y_continuous(name=NULL, expand=c(1,1)/100) +
  scale_linetype_manual("", values=c("mean"="solid", "true value"="dotted")) +
  scale_fill_manual("", values="grey") +
  guides(linetype=guide_legend(nrow=1), fill=guide_legend(nrow=1)) +
  facet_grid(`event time`~., labeller=label_both) +
  theme_classic() + mytheme +
  theme(plot.title=element_text(hjust=0),
        legend.position="top", legend.box="horizontal")

ggpubr::ggarrange(accplt.pd, accplt.cal, common.legend=T, ncol=2)

ggsave(paste0("acc_pd_cal_mean_95percentile_n=10000_pen_mu1_0.005_mu2_0.002_", Sys.Date(), ".pdf"), width=7, height=9)
