library(tidyverse)
path.input <- file.path("readmission2020_ce_mm18_7cat.csv")
options(datatable.fread.datatable=F)
Rcpp::sourceCpp("surtiver_tps.cpp")
source("surtiver_tps_cv.R")
mytheme <- theme(plot.title=element_text(hjust=0),
                 axis.line=element_line(color="black"),
                 axis.title=element_text(size=18),
                 axis.text=element_text(size=12),
                 text=element_text(size=12),
                 legend.title=element_blank(),
                 legend.text=element_text(size=14),
                 strip.text=element_text(size=14),
                 legend.position="none")
mytheme2 <- theme(plot.title=element_text(hjust=0),
                  axis.line=element_line(color="black"),
                  axis.title=element_text(size=18),
                  axis.text=element_text(size=12),
                  text=element_text(size=12),
                  legend.title=element_text(size=18),
                  legend.text=element_text(size=14),
                  strip.text=element_text(size=14),
                  legend.position=c(0.8, 0.9))
mytheme3 <- theme(plot.title=element_text(hjust=0),
                  axis.line=element_line(color="black"),
                  axis.title=element_text(size=18),
                  axis.text=element_text(size=12),
                  text=element_text(size=12),
                  legend.title=element_text(size=18),
                  legend.text=element_text(size=14),
                  strip.text=element_text(size=14),
                  legend.position=c(0.2, 0.9))
### unplanned hospital readmission
suppressWarnings(
  data <- data.table::fread(path.input, header=T, na.strings=c(".","na","")))
data <- data.table::fread(path.input, colClasses=unname(sapply(data, class)), 
                          header=T, na.strings=c(".","na",""))
disc.status <- "discharge_status_7cat"
covid <- "covid_flag"
fac.char <- "provfs"
timetoevents <- "days_to_ce"
type <- "ce_type"; race <- "RACEGRPC"
UHR <- "readmission"; death <- "death"
time.pd <- "days_to_ce"
time.since.onset <- "time_since_onset"
date.onset <- as.Date("2020-03-01")
data <- data %>%
  mutate(!!(type):=replace_na(!!sym(type), 9),
         !!(UHR):=1*(!!sym(type)==0),
         !!(death):=1*(!!sym(type)==2),
         black=1*(!!sym(race)=="Black"),
         other_race=1*(!!sym(race)%in%c("Black", "White")),
         nonacute=1*(!!sym(disc.status)==4),
         home_med=1*(!!sym(disc.status)==6),
         SNF=1*(!!sym(disc.status)==7),
         other_dis=1*(!!sym(disc.status)%in%c(1:3))) %>%
  rename(female=sex)
data <- data %>%
  mutate(!!(time.since.onset):=as.numeric(as.Date(CLM_THRU, "%d%B%Y")-date.onset)) %>%
  filter(!!sym(time.since.onset)>=0)

covid <- "covid_flag"
covar <- c("female","agelt25","age25_45","age60_75","agegt75",
           "black","other_race","bmi_under","bmi_over","bmi_obese",
           "vincat2", "vincat3", "vincat4", "vincat5",
           "MA_ind","NH_lt90","NH_ge90",
           "planned", "nonacute", "home_med", "SNF", "other_dis",
           "timeinhosp_quantile2","timeinhosp_quantile3","timeinhosp_quantile4",
           "risky_currentdx", "esrdcause_diab",
           "ahrq_6","ahrq_10", "ahrq_42", "ahrq_50", "ahrq_51", "ahrq_52", 
           "ahrq_55", "ahrq_59", "ahrq_64", "ahrq_95", "ahrq_96", "ahrq_97", 
           "ahrq_100", "ahrq_101", "ahrq_102", "ahrq_106", "ahrq_107", 
           "ahrq_108", "ahrq_117", "ahrq_118", "ahrq_120", "ahrq_121", 
           "ahrq_122", "ahrq_127", "ahrq_130", "ahrq_131", "ahrq_133",
           "ahrq_134", "ahrq_135", "ahrq_138", "ahrq_140", "ahrq_141",
           "ahrq_151", "ahrq_152", "ahrq_153", "ahrq_154", "ahrq_155",
           "ahrq_158", "ahrq_159", "ahrq_197", "ahrq_198", "ahrq_199",
           "ahrq_201", "ahrq_237", "ahrq_244", "ahrq_251", "ahrq_253",
           "ahrq_255", "ahrq_259", "ahrq_651", "ahrq_659", "ahrq_660", "ahrq_661")
level <- 0.95
quant.upper <- qnorm((1+level)/2)
data <- data[order(data[,fac.char], data[,timetoevents], -data[,UHR]),]
times <- data[,timetoevents]; times <- unique(times); times <- times[order(times)]
strata.noevent <-
  unique(data[,fac.char])[sapply(split(data[,UHR],data[,fac.char]), sum)==0]
data <- data[!data[,fac.char]%in%strata.noevent,] # drop strata with no event
data <- data %>% filter(!!sym(fac.char)%in%unique(data$provfs))
count.strata <- sapply(split(data[,fac.char], data[,fac.char]), length)
nsplines1 <- 8; degree1 <- 3
knots.B1 <-
  quantile(data[,time.since.onset],
           (1:(nsplines1-degree1-1))/(nsplines1-degree1))
bases1 <- splines::bs(data[,time.since.onset], degree=degree1, intercept=T, 
                      knots=knots.B1)
boundaries1 <- range(data[,time.since.onset])
nsplines2 <- 6; degree2 <- 2
knots.B2 <- 
  quantile(data[data[,UHR]==1,timetoevents],
           (1:(nsplines2-degree2-1))/(nsplines2-degree2))
uniqfailtimes.str <- unname(unlist(lapply(
  split(data[data[,UHR]==1,timetoevents],
        data[data[,UHR]==1,fac.char]), unique)))
bases2 <- splines::bs(uniqfailtimes.str, degree=degree2, intercept=T,
                     knots=knots.B2, Boundary.knots=range(times))
boundaries2 <- range(uniqfailtimes.str)
set.seed(1)
theta.init <- matrix(runif(length(covid)*nsplines1*nsplines2),
                     length(covid), nsplines1*nsplines2)
beta.ti.init <- matrix(runif(length(covar)),length(covar))

Wald.test <- function(theta.vec, contrast, mat.var) {
  pchisq(t(theta.vec)%*%t(contrast)%*%
           solve(contrast%*%mat.var%*%t(contrast))%*%
           contrast%*%theta.vec, nrow(contrast), lower=F)
}
system.time(fit.UHR.unpen <-
              surtiver_fixtra_bresties_fit(data[,UHR], data[,timetoevents],
                                           count.strata, 
                                           as.matrix(data[,covid]),
                                           as.matrix(bases1),
                                           as.matrix(bases2), 
                                           theta.init,
                                           as.matrix(data[,covar]), 
                                           beta.ti.init))
D1 <- diff(diag(nsplines1)); D2 <- diff(diag(nsplines2))
fit.cv.UHR <-
  cv.surbiver(data, UHR, timetoevents, time.since.onset, fac.char,
              covid, covar, parallel=T)
lambda.time <- fit.cv.UHR %>% slice_min(cve) %>% pull(lambda.time)
lambda.var <- fit.cv.UHR %>% slice_min(cve) %>% pull(lambda.var)
fit.cv2.UHR <-
  expand_grid(lambda.time=10^seq(log10(lambda.time)-0.4, log10(lambda.time)+0.5, 0.1),
              lambda.var=10^seq(log10(lambda.var)-0.4, log10(lambda.var)+0.5, 0.1)) %>%
  cv.surbiver(data, UHR, timetoevents, time.since.onset, fac.char, covid, covar,
              grid=., parallel=T)
mu1 <- fit.cv2.UHR %>% slice_min(cve) %>% pull(lambda.time)
mu2 <- fit.cv2.UHR %>% slice_min(cve) %>% pull(lambda.var)
system.time(fit.UHR.diffpen <-
              surtiver_fixtra_bresties_fit_diffpen(data[,UHR], data[,timetoevents],
                                                   count.strata, 
                                                   as.matrix(data[,covid]),
                                                   as.matrix(bases1),
                                                   as.matrix(bases2), 
                                                   theta.init,
                                                   as.matrix(data[,covar]), 
                                                   beta.ti.init,
                                                   D1, D2, mu1, mu2))
invinfo.UHR.diffpen <-
  solve(fit.UHR.diffpen$info[1:(nsplines1*nsplines2), 1:(nsplines1*nsplines2)])
ctrast.pd <- as.matrix(do.call(Matrix::bdiag, rep(list(diff(diag(nsplines2))), nsplines1)))
ctrast.cal <- diff(diag(nsplines1*nsplines2), nsplines2)
ctrast.pdcal <- diff(diag(nsplines1*nsplines2))
p.pd <- Wald.test(fit.UHR.diffpen$ctrl.pts[1,], ctrast.pd, invinfo.UHR.diffpen)
p.cal <- Wald.test(fit.UHR.diffpen$ctrl.pts[1,], ctrast.cal, invinfo.UHR.diffpen)
p.pdcal <- Wald.test(fit.UHR.diffpen$ctrl.pts[1,], ctrast.pdcal, invinfo.UHR.diffpen)
c(p.pd, p.cal, p.pdcal)
# post-discharge variation by month
dates.B1 <- seq(as.Date("2020-04-01"), as.Date("2020-10-15"), by="2 months")
t.B1 <- as.numeric(dates.B1-date.onset)
t.B2 <- 0:30
B1 <- as.matrix(splines::bs(t.B1, degree=degree1, intercept=T, knots=knots.B1,
                    Boundary.knots=boundaries1))
B2 <- splines::bs(t.B2, degree=degree2, intercept=T,
                  knots=knots.B2, Boundary.knots=boundaries2)
mat.eff <- B1%*%matrix(fit.UHR.diffpen$ctrl.pts[1,], byrow=T, ncol=nsplines2)%*%t(B2)
var.tv <- solve(fit.UHR.diffpen$info)[1:(nsplines1*nsplines2),1:(nsplines1*nsplines2)]
B1.expand <- B1[rep(1:nrow(B1), each=nrow(B2)),]
B2.expand <- kronecker(matrix(1, nrow(B1), 1), B2)
mat.dev <- matrix(sapply(1:(nrow(B1)*nrow(B2)),
                         function(i) {
                           tmp <- kronecker(B1.expand[i,], B2.expand[i,])
                           sqrt(t(tmp)%*%var.tv%*%tmp)
                         })*quant.upper, nrow=length(t.B1), byrow=T)
mat.lower <- mat.eff - mat.dev
mat.upper <- mat.eff + mat.dev
dimnames(mat.eff) <- dimnames(mat.lower) <- dimnames(mat.upper) <-
  list(as.character(dates.B1), t.B2)
mat.eff <- t(mat.eff); mat.lower <- t(mat.lower); mat.upper <- t(mat.upper)
tbl.eff <- as_tibble(mat.eff) %>% mutate(`days after discharge`=t.B2) %>%
  pivot_longer(!`days after discharge`, names_to="date", values_to="log hazard ratio") %>%
  mutate(
    lower=as_tibble(mat.lower) %>% mutate(`days after discharge`=t.B2) %>%
      pivot_longer(!`days after discharge`, names_to="date", values_to="lower") %>% pull(lower),
    upper=as_tibble(mat.upper) %>% mutate(`days after discharge`=t.B2) %>%
      pivot_longer(!`days after discharge`, names_to="date", values_to="upper") %>% pull(upper))
labels.pd <- format(seq(as.Date("2020-04-01"), as.Date("2020-10-01"), by="2 months"), format="%b %d")
UHR.eff.pd <- tbl.eff %>%
  ggplot(aes(x=`days after discharge`, y=`log hazard ratio`, group=date,
             linetype=date, color=date)) +
  geom_ribbon(aes(ymin=lower, ymax=upper, fill=date), color=NA, alpha=0.1) +
  geom_line(size=0.9) +
  scale_linetype_manual("date of discharge", values=c("solid", "dotted", "dashed", "dotdash"),
                        labels=labels.pd) +
  scale_color_manual("date of discharge", values=c("red","blue","green4","black"),
                     labels=labels.pd) +
  scale_fill_manual("date of discharge", values=c("red","blue","green4","black"), labels=labels.pd) +
  guides(linetype=guide_legend(nrow=1), fill=guide_legend(nrow=1), color=guide_legend(nrow=1)) +
  geom_hline(yintercept=0, linetype="dashed") +
  labs(title="postdischarge readmission") +
  scale_x_continuous(expand=c(1,1)/100) +
  scale_y_continuous(expand=c(1,1)/100, limits=c(-1,1.8)) +
  theme_classic() + mytheme2
# calendar variation by post-discharge day
dates.B1 <- seq(as.Date("2020-04-01"), as.Date("2020-10-15"), by="day")
t.B1 <- as.numeric(dates.B1-date.onset)
t.B2 <- c(3,5,15,25)
B1 <- as.matrix(splines::bs(t.B1, degree=degree1, intercept=T, knots=knots.B1,
                            Boundary.knots=boundaries1))
B2 <- splines::bs(t.B2, degree=degree2, intercept=T,
                  knots=knots.B2, Boundary.knots=boundaries2)
mat.eff <- B1%*%matrix(fit.UHR.diffpen$ctrl.pts[1,], byrow=T, ncol=nsplines2)%*%t(B2)
var.tv <- solve(fit.UHR.diffpen$info)[1:(nsplines1*nsplines2), 1:(nsplines1*nsplines2)]
B1.expand <- B1[rep(1:nrow(B1), each=nrow(B2)),]
B2.expand <- kronecker(matrix(1, nrow(B1), 1), B2)
mat.dev <- matrix(sapply(1:(nrow(B1)*nrow(B2)),
                         function(i) {
                           tmp <- kronecker(B1.expand[i,], B2.expand[i,])
                           sqrt(t(tmp)%*%var.tv%*%tmp)
                         })*quant.upper, nrow=length(t.B1), byrow=T)
mat.lower <- mat.eff - mat.dev
mat.upper <- mat.eff + mat.dev
dimnames(mat.eff) <- dimnames(mat.lower) <- dimnames(mat.upper) <-
  list(as.character(dates.B1), t.B2)
tbl.eff <- as_tibble(mat.eff) %>% mutate(`date of discharge`=t.B1) %>%
  pivot_longer(!`date of discharge`, names_to="date", values_to="log hazard ratio") %>%
  mutate(
    lower=as_tibble(mat.lower) %>% mutate(`date of discharge`=t.B1) %>%
      pivot_longer(!`date of discharge`, names_to="date", values_to="lower") %>% pull(lower),
    upper=as_tibble(mat.upper) %>% mutate(`date of discharge`=t.B1) %>%
      pivot_longer(!`date of discharge`, names_to="date", values_to="upper") %>% pull(upper))
labels <- format(seq(as.Date("2020-04-01"), as.Date("2020-10-01"), by="month"), format="%b %d")
breaks <- as.numeric(seq(as.Date("2020-04-01"), as.Date("2020-10-01"), by="month") - date.onset)
UHR.eff.cal <- tbl.eff %>% mutate(date=factor(date, levels=c(3,5,15,25))) %>%
  ggplot(aes(x=`date of discharge`, y=`log hazard ratio`, group=date,
             linetype=date, color=date)) +
  geom_ribbon(aes(ymin=lower, ymax=upper, fill=date), color=NA, alpha=0.1) +
  geom_line(size=0.9) +
  scale_linetype_manual("days since discharge", values=c("solid", "dotted", "dashed", "dotdash")) +
  scale_color_manual("days since discharge", values=c("red","blue","green4","black")) +
  scale_fill_manual("days since discharge", values=c("red","blue","green4","black")) +
  guides(linetype=guide_legend(nrow=1), fill=guide_legend(nrow=1), color=guide_legend(nrow=1)) +
  geom_hline(yintercept=0, linetype="dashed") +
  scale_x_continuous(expand=c(1,1)/100, breaks=breaks, labels=labels) +
  scale_y_continuous(expand=c(1,1)/100, limits=c(-1,1.8)) +
  labs(title="postdischarge readmission") +
  theme_classic() + mytheme3
# bivariate variation
library(plotly)
dates.B1 <- seq(as.Date("2020-04-01"), as.Date("2020-10-15"), by="day")
t.B1 <- as.numeric(dates.B1-date.onset)
t.B2 <- 0:30
B1 <- as.matrix(splines::bs(t.B1,
                            degree=degree1, intercept=T, knots=knots.B1,
                            Boundary.knots=boundaries1))
B2 <- splines::bs(t.B2, degree=degree2, intercept=T,
                  knots=knots.B2, Boundary.knots=boundaries2)
mat.eff.UHR <- B1%*%matrix(fit.UHR.diffpen$ctrl.pts[1,], byrow=T, ncol=nsplines2)%*%t(B2)
dimnames(mat.eff.UHR) <- list(as.character(dates.B1), t.B2)
mat.eff.UHR <- t(mat.eff.UHR)
xaxis <-
  list(title='date of discharge',
       automargin=T,
       ticktext=format(seq(as.Date("2020-04-01"), as.Date("2020-10-01"), by="month"), format="%b %d"),
       tickangle=0,
       tickvals=as.numeric(
         seq(as.Date("2020-04-01"), as.Date("2020-10-01"), by="month")-as.Date("2020-04-01")))
yaxis <- list(title='days since discharge', automargin=T)
scene.eff <- list(xaxis=xaxis, yaxis=yaxis,
                  zaxis=list(title='log hazard ratio'),
                  camera=list(eye=list(x=2/1.3, y=2/1.3, z=0.6/1.3)))

UHR3d.eff <- plot_ly(z=mat.eff.UHR, colors=c("blue","green","yellow","orange","red")) %>%
  layout(scene=scene.eff) %>% add_surface(colorbar=list(title='log HR', x=1, y=0.6))

xaxis.UHR.contour <-
  list(title='date of discharge',
       ticktext=format(seq(as.Date("2020-04-01"), as.Date("2020-10-01"), by="month"), format="%b %d"),
       tickvals=as.numeric(
         seq(as.Date("2020-04-01"), as.Date("2020-10-01"), by="month")-as.Date("2020-04-01")))
yaxis.UHR.contour <- list(title='days since discharge')

UHR.contour.eff <-
  plot_ly(z=mat.eff.UHR, colors=c("blue","green","yellow","orange","red")) %>%
  layout(xaxis=xaxis.UHR.contour, yaxis=yaxis.UHR.contour) %>%
  add_contour(colorbar=list(title='log HR'), contours=list(showlabels=T))

### death
suppressWarnings(
  data <- data.table::fread(path.input, header=T, na.strings=c(".","na","")))
data <- data.table::fread(path.input, colClasses=unname(sapply(data, class)), 
                          header=T, na.strings=c(".","na",""))
disc.status <- "discharge_status_7cat"
covid <- "covid_flag"
fac.char <- "provfs"
timetoevents <- "days_to_ce"
type <- "ce_type"; race <- "RACEGRPC"
UHR <- "readmission"; death <- "death"
time.pd <- "days_to_ce"
time.since.onset <- "time_since_onset"
date.onset <- as.Date("2020-03-01")
data <- data %>%
  mutate(!!(type):=replace_na(!!sym(type), 9),
         !!(UHR):=1*(!!sym(type)==0),
         !!(death):=1*(!!sym(type)==2),
         black=1*(!!sym(race)=="Black"),
         other_race=1*(!!sym(race)%in%c("Black", "White")),
         nonacute=1*(!!sym(disc.status)==4),
         home_med=1*(!!sym(disc.status)==6),
         SNF=1*(!!sym(disc.status)==7),
         other_dis=1*(!!sym(disc.status)%in%c(1:3))) %>%
  rename(female=sex)
data <- data %>%
  mutate(!!(time.since.onset):=as.numeric(as.Date(CLM_THRU, "%d%B%Y")-date.onset)) %>%
  filter(!!sym(time.since.onset)>=0)

covid <- "covid_flag"
covar <- c("female","agelt25","age25_45","age60_75","agegt75",
           "black","other_race","bmi_under","bmi_over","bmi_obese",
           "vincat2", "vincat3", "vincat4", "vincat5",
           "MA_ind","NH_lt90","NH_ge90",
           "planned", "nonacute", "home_med", "SNF", "other_dis",
           "timeinhosp_quantile2","timeinhosp_quantile3","timeinhosp_quantile4",
           "risky_currentdx", "esrdcause_diab",
           "ahrq_6","ahrq_10", "ahrq_42", "ahrq_50", "ahrq_51", "ahrq_52", 
           "ahrq_55", "ahrq_59", "ahrq_64", "ahrq_95", "ahrq_96", "ahrq_97", 
           "ahrq_100", "ahrq_101", "ahrq_102", "ahrq_106", "ahrq_107", 
           "ahrq_108", "ahrq_117", "ahrq_118", "ahrq_120", "ahrq_121", 
           "ahrq_122", "ahrq_127", "ahrq_130", "ahrq_131", "ahrq_133",
           "ahrq_134", "ahrq_135", "ahrq_138", "ahrq_140", "ahrq_141",
           "ahrq_151", "ahrq_152", "ahrq_153", "ahrq_154", "ahrq_155",
           "ahrq_158", "ahrq_159", "ahrq_197", "ahrq_198", "ahrq_199",
           "ahrq_201", "ahrq_237", "ahrq_244", "ahrq_251", "ahrq_253",
           "ahrq_255", "ahrq_259", "ahrq_651", "ahrq_659", "ahrq_660", "ahrq_661")

data <- data[order(data[,fac.char], data[,timetoevents], -data[,death]),]
times <- data[,timetoevents]; times <- unique(times); times <- times[order(times)]
strata.noevent <-
  unique(data[,fac.char])[sapply(split(data[,death],data[,fac.char]), sum)==0]
data <- data[!data[,fac.char]%in%strata.noevent,] # drop strata with no event
data <- data %>% filter(!!sym(fac.char)%in%unique(data$provfs))
count.strata <- sapply(split(data[,fac.char], data[,fac.char]), length)
nsplines1 <- 6; degree1 <- 3
knots.B1 <-
  quantile(data[,time.since.onset],
           (1:(nsplines1-degree1-1))/(nsplines1-degree1))
bases1 <- splines::bs(data[,time.since.onset], degree=degree1, intercept=T, 
                      knots=knots.B1)
boundaries1 <- range(data[,time.since.onset])
nsplines2 <- 6; degree2 <- 3
knots.B2 <-
  quantile(data[data[,death]==1,timetoevents],
           (1:(nsplines2-degree2-1))/(nsplines2-degree2))
uniqfailtimes.str <- unname(unlist(lapply(
  split(data[data[,death]==1,timetoevents],
        data[data[,death]==1,fac.char]), unique)))
bases2 <- splines::bs(uniqfailtimes.str, degree=degree2, intercept=T,
                      knots=knots.B2, Boundary.knots=range(times))
boundaries2 <- range(uniqfailtimes.str)
set.seed(1)
theta.init <- matrix(runif(length(covid)*nsplines1*nsplines2),
                     length(covid), nsplines1*nsplines2)
beta.ti.init <- matrix(runif(length(covar)),length(covar))
system.time(fit.death.unpen <-
              surtiver_fixtra_bresties_fit(data[,death], data[,timetoevents],
                                           count.strata, 
                                           as.matrix(data[,covid]),
                                           as.matrix(bases1),
                                           as.matrix(bases2), 
                                           theta.init,
                                           as.matrix(data[,covar]), 
                                           beta.ti.init))

D1 <- diff(diag(nsplines1)); D2 <- diff(diag(nsplines2))
fit.cv.death <-
  cv.surbiver(data, death, timetoevents, time.since.onset, fac.char,
              covid, covar, parallel=T)
lambda.time <- fit.cv.death %>% slice_min(cve) %>% pull(lambda.time)
lambda.var <- fit.cv.death %>% slice_min(cve) %>% pull(lambda.var)
fit.cv2.death <-
  expand_grid(lambda.time=10^seq(log10(lambda.time)-0.4, log10(lambda.time)+0.5, 0.1),
              lambda.var=10^seq(log10(lambda.var)-0.4, log10(lambda.var)+0.5, 0.1)) %>%
  cv.surbiver(data, death, timetoevents, time.since.onset, fac.char, covid, covar,
              grid=., parallel=T)
mu1 <- fit.cv2.death %>% slice_min(cve) %>% pull(lambda.time)
mu2 <- fit.cv2.death %>% slice_min(cve) %>% pull(lambda.var)
system.time(fit.death.diffpen <-
              surtiver_fixtra_bresties_fit_diffpen(data[,death], data[,timetoevents],
                                                   count.strata, 
                                                   as.matrix(data[,covid]),
                                                   as.matrix(bases1),
                                                   as.matrix(bases2), 
                                                   theta.init,
                                                   as.matrix(data[,covar]), 
                                                   beta.ti.init,
                                                   D1, D2, mu1, mu2))

invinfo.death.diffpen <-
  solve(fit.death.diffpen$info[1:(nsplines1*nsplines2), 1:(nsplines1*nsplines2)])
ctrast.pd <- as.matrix(do.call(Matrix::bdiag, rep(list(diff(diag(nsplines2))), nsplines1)))
ctrast.cal <- diff(diag(nsplines1*nsplines2), nsplines2)
ctrast.pdcal <- diff(diag(nsplines1*nsplines2))
p.pd <- Wald.test(fit.death.diffpen$ctrl.pts[1,], ctrast.pd, invinfo.death.diffpen)
p.cal <- Wald.test(fit.death.diffpen$ctrl.pts[1,], ctrast.cal, invinfo.death.diffpen)
p.pdcal <- Wald.test(fit.death.diffpen$ctrl.pts[1,], ctrast.pdcal, invinfo.death.diffpen)
c(p.pd, p.cal, p.pdcal)

# post-discharge variation by month
dates.B1 <- seq(as.Date("2020-04-01"), as.Date("2020-10-01"), by="2 months")
t.B1 <- as.numeric(dates.B1-date.onset)
t.B2 <- 1:30
B1 <- as.matrix(splines::bs(t.B1, degree=degree1, intercept=T, knots=knots.B1,
                            Boundary.knots=boundaries1))
B2 <- splines::bs(t.B2, degree=degree2, intercept=T,
                  knots=knots.B2, Boundary.knots=boundaries2)
mat.eff <- B1%*%matrix(fit.death.diffpen$ctrl.pts[1,], byrow=T, ncol=nsplines2)%*%t(B2)
var.tv <- solve(fit.death.diffpen$info)[1:(nsplines1*nsplines2), 1:(nsplines1*nsplines2)]
B1.expand <- B1[rep(1:nrow(B1), each=nrow(B2)),]
B2.expand <- kronecker(matrix(1, nrow(B1), 1), B2)
mat.dev <- matrix(sapply(1:(nrow(B1)*nrow(B2)),
                         function(i) {
                           tmp <- kronecker(B1.expand[i,], B2.expand[i,])
                           sqrt(t(tmp)%*%var.tv%*%tmp)
                         })*quant.upper, nrow=length(t.B1), byrow=T)
mat.lower <- mat.eff - mat.dev
mat.upper <- mat.eff + mat.dev
dimnames(mat.eff) <- dimnames(mat.lower) <- dimnames(mat.upper) <-
  list(as.character(dates.B1), t.B2)
mat.eff <- t(mat.eff); mat.lower <- t(mat.lower); mat.upper <- t(mat.upper)
tbl.eff <- as_tibble(mat.eff) %>% mutate(`days after discharge`=t.B2) %>%
  pivot_longer(!`days after discharge`, names_to="date", values_to="log hazard ratio") %>%
  mutate(
    lower=as_tibble(mat.lower) %>% mutate(`days after discharge`=t.B2) %>%
      pivot_longer(!`days after discharge`, names_to="date", values_to="lower") %>% pull(lower),
    upper=as_tibble(mat.upper) %>% mutate(`days after discharge`=t.B2) %>%
      pivot_longer(!`days after discharge`, names_to="date", values_to="upper") %>% pull(upper))
death.eff.pd <- tbl.eff %>%
  ggplot(aes(x=`days after discharge`, y=`log hazard ratio`, group=date,
             linetype=date, color=date)) +
  geom_ribbon(aes(ymin=lower, ymax=upper, fill=date), color=NA, alpha=0.1) +
  geom_line(size=0.9) +
  scale_linetype_manual("date of discharge", values=c("solid", "dotted", "dashed", "dotdash"),
                        labels=labels.pd) +
  scale_color_manual("date of discharge", values=c("red","blue","green4","black"),
                     labels=labels.pd) +
  scale_fill_manual("date of discharge", values=c("red","blue","green4","black"), labels=labels.pd) +
  guides(linetype=guide_legend(nrow=1), fill=guide_legend(nrow=1), color=guide_legend(nrow=1)) +
  geom_hline(yintercept=0, linetype="dashed") +
  labs(title="postdischarge death") +
  scale_x_continuous(expand=c(1,1)/100) +
  scale_y_continuous(expand=c(1,1)/100, limits=c(-1,1.8)) +
  theme_classic() + mytheme2

# calendar variation by post-discharge day
dates.B1 <- seq(as.Date("2020-04-01"), as.Date("2020-10-15"), by="day")
t.B1 <- as.numeric(dates.B1-date.onset)
t.B2 <- c(3,5,15,25)
B1 <- as.matrix(splines::bs(t.B1, degree=degree1, intercept=T, knots=knots.B1,
                            Boundary.knots=boundaries1))
B2 <- splines::bs(t.B2, degree=degree2, intercept=T,
                  knots=knots.B2, Boundary.knots=boundaries2)
mat.eff <- B1%*%matrix(fit.death.diffpen$ctrl.pts[1,], byrow=T, ncol=nsplines2)%*%t(B2)
var.tv <- solve(fit.death.diffpen$info)[1:(nsplines1*nsplines2), 1:(nsplines1*nsplines2)]
B1.expand <- B1[rep(1:nrow(B1), each=nrow(B2)),]
B2.expand <- kronecker(matrix(1, nrow(B1), 1), B2)
mat.dev <- matrix(sapply(1:(nrow(B1)*nrow(B2)),
                         function(i) {
                           tmp <- kronecker(B1.expand[i,], B2.expand[i,])
                           sqrt(t(tmp)%*%var.tv%*%tmp)
                         })*quant.upper, nrow=length(t.B1), byrow=T)
mat.lower <- mat.eff - mat.dev
mat.upper <- mat.eff + mat.dev
dimnames(mat.eff) <- dimnames(mat.lower) <- dimnames(mat.upper) <-
  list(as.character(dates.B1), t.B2)
tbl.eff <- as_tibble(mat.eff) %>% mutate(`date of discharge`=t.B1) %>%
  pivot_longer(!`date of discharge`, names_to="date", values_to="log hazard ratio") %>%
  mutate(
    lower=as_tibble(mat.lower) %>% mutate(`date of discharge`=t.B1) %>%
      pivot_longer(!`date of discharge`, names_to="date", values_to="lower") %>% pull(lower),
    upper=as_tibble(mat.upper) %>% mutate(`date of discharge`=t.B1) %>%
      pivot_longer(!`date of discharge`, names_to="date", values_to="upper") %>% pull(upper))
labels <- format(seq(as.Date("2020-04-01"), as.Date("2020-10-01"), by="month"), format="%b %d")
breaks <- as.numeric(seq(as.Date("2020-04-01"), as.Date("2020-10-01"), by="month") - date.onset)
death.eff.cal <- tbl.eff %>% mutate(date=factor(date, levels=c(3,5,15,25))) %>%
  ggplot(aes(x=`date of discharge`, y=`log hazard ratio`, group=date,
             linetype=date, color=date)) +
  geom_ribbon(aes(ymin=lower, ymax=upper, fill=date), color=NA, alpha=0.1) +
  geom_line(size=0.9) +
  scale_linetype_manual("days since discharge", values=c("solid", "dotted", "dashed", "dotdash")) +
  scale_color_manual("days since discharge", values=c("red","blue","green4","black")) +
  scale_fill_manual("days since discharge", values=c("red","blue","green4","black")) +
  guides(linetype=guide_legend(nrow=1), fill=guide_legend(nrow=1), color=guide_legend(nrow=1)) +
  geom_hline(yintercept=0, linetype="dashed") +
  labs(title="postdischarge death") +
  scale_x_continuous(expand=c(1,1)/100, breaks=breaks, labels=labels) +
  scale_y_continuous(expand=c(1,1)/100, limits=c(-1,1.8)) +
  theme_classic() + mytheme3

# bivariate variation
library(plotly)
dates.B1 <- seq(as.Date("2020-04-01"), as.Date("2020-10-15"), by="day")
t.B1 <- as.numeric(dates.B1-date.onset)
t.B2 <- 1:30
B1 <- as.matrix(splines::bs(t.B1,
                            degree=degree1, intercept=T, knots=knots.B1,
                            Boundary.knots=boundaries1))
B2 <- splines::bs(t.B2, degree=degree2, intercept=T,
                  knots=knots.B2, Boundary.knots=boundaries2)
mat.eff.death <- B1%*%matrix(fit.death.diffpen$ctrl.pts[1,], byrow=T, ncol=nsplines2)%*%t(B2)
dimnames(mat.eff.death) <- list(as.character(dates.B1), t.B2)
mat.eff.death <- t(mat.eff.death)
xaxis <-
  list(title='date of discharge',
       automargin=T,
       ticktext=format(seq(as.Date("2020-04-01"), as.Date("2020-10-01"), by="month"), format="%b %d"),
       tickangle=0,
       tickvals=as.numeric(
         seq(as.Date("2020-04-01"), as.Date("2020-10-01"), by="month")-as.Date("2020-04-01")))
yaxis <- list(title='days since discharge',
              ticktext=c(0,4,9,14,19,24,29)+1,
              tickangle=0,
              tickvals=c(0,4,9,14,19,24,29),
              automargin=T)
scene.eff <- list(xaxis=xaxis, yaxis=yaxis,
                  zaxis=list(title='log hazard ratio'),
                  camera=list(eye=list(x=2/1.3, y=2/1.3, z=0.6/1.3)))
death3d.eff <- plot_ly(z=mat.eff.death, colors=c("blue","green","yellow","orange","red")) %>%
  layout(scene=scene.eff) %>% add_surface(colorbar=list(title='log HR', x=1, y=0.6))

death.contour.eff <-
  plot_ly(z=mat.eff.death, colors=c("blue","green","yellow","orange","red")) %>%
  layout(xaxis=xaxis, yaxis=yaxis) %>%
  add_contour(colorbar=list(title='log HR'), contours=list(showlabels=T))